@@ -18,9 +18,70 @@ from supervised.utils.config import LOG_LEVEL
 logger = logging.getLogger(__name__)
 logger.setLevel(LOG_LEVEL)
 
-
+from sklearn.tree import _tree
 from dtreeviz.trees import dtreeviz
 
+def get_rules(tree, feature_names, class_names):
+    tree_ = tree.tree_
+    feature_name = [
+        feature_names[i] if i != _tree.TREE_UNDEFINED else "undefined!"
+        for i in tree_.feature
+    ]
+
+    paths = []
+    path = []
+    
+    def recurse(node, path, paths):
+        
+        if tree_.feature[node] != _tree.TREE_UNDEFINED:
+            name = feature_name[node]
+            threshold = tree_.threshold[node]
+            p1, p2 = list(path), list(path)
+            p1 += [f"({name} <= {np.round(threshold, 3)})"]
+            recurse(tree_.children_left[node], p1, paths)
+            p2 += [f"({name} > {np.round(threshold, 3)})"]
+            recurse(tree_.children_right[node], p2, paths)
+        else:
+            path += [(tree_.value[node], tree_.n_node_samples[node])]
+            paths += [path]
+            
+    recurse(0, path, paths)
+
+    # sort by samples count
+    samples_count = [p[-1][1] for p in paths]
+    ii = list(np.argsort(samples_count))
+    paths = [paths[i] for i in reversed(ii)]
+    
+    rules = []
+    for path in paths:
+        rule = "if "
+        
+        for p in path[:-1]:
+            if rule != "if ":
+                rule += " and "
+            rule += str(p)
+        rule += " then "
+        if class_names is None:
+            rule += "response: "+str(np.round(path[-1][0][0][0],3))
+        else:
+            classes = path[-1][0][0]
+            l = np.argmax(classes)
+            rule += f"class: {class_names[l]} (proba: {np.round(100.0*classes[l]/np.sum(classes),2)}%)"
+        rule += f" | based on {path[-1][1]:,} samples"
+        rules += [rule]
+        
+    return rules
+
+
+def save_rules(tree, feature_names, class_names, model_file_path, learner_name):
+    try:
+        rules = get_rules(tree, feature_names, class_names)
+        fname = os.path.join(model_file_path, f"{learner_name}_rules.txt")
+        with open(fname, "w") as fout:
+            for r in rules:
+                fout.write(r + "\n\n")
+    except Exception as e:
+        logger.info(f"Problem with extracting decision tree rules. {str(e)}")
 
 class DecisionTreeAlgorithm(SklearnAlgorithm):
 
@@ -87,6 +148,9 @@ class DecisionTreeAlgorithm(SklearnAlgorithm):
         except Exception as e:
             logger.info(f"Problem when visualizing decision tree. {str(e)}")
 
+        save_rules(self.model, X_train.columns, class_names, model_file_path, learner_name)
+
+
 
 class DecisionTreeRegressorAlgorithm(SklearnAlgorithm):
 
@@ -150,6 +214,8 @@ class DecisionTreeRegressorAlgorithm(SklearnAlgorithm):
         except Exception as e:
             logger.info(f"Problem when visuzalizin decision tree regressor. {str(e)}")
 
+        save_rules(self.model, X_train.columns, None, model_file_path, learner_name)
+
 
 dt_params = {"criterion": ["gini", "entropy"], "max_depth": [2, 3, 4]}
 
