@@ -40,13 +40,16 @@ class ExtraTreesAlgorithm(SklearnTreesEnsembleClassifierAlgorithm):
             max_depth=params.get("max_depth", 6),
             min_samples_split=params.get("min_samples_split", 4),
             warm_start=True,
-            n_jobs=-1,
+            n_jobs=params.get("n_jobs", -1),
             random_state=params.get("seed", 1),
         )
 
     def file_extension(self):
         return "extra_trees"
 
+    def get_metric_name(self):
+        return "logloss"
+
 
 class ExtraTreesRegressorAlgorithm(SklearnTreesEnsembleRegressorAlgorithm):
 
@@ -70,13 +73,16 @@ class ExtraTreesRegressorAlgorithm(SklearnTreesEnsembleRegressorAlgorithm):
             max_depth=params.get("max_depth", 6),
             min_samples_split=params.get("min_samples_split", 30),
             warm_start=True,
-            n_jobs=-1,
+            n_jobs=params.get("n_jobs", -1),
             random_state=params.get("seed", 1),
         )
 
     def file_extension(self):
         return "extra_trees"
 
+    def get_metric_name(self):
+        return "rmse"
+
 
 # For binary classification target should be 0, 1. There should be no NaNs in target.
 et_params = {
