@@ -5,7 +5,6 @@ import pandas as pd
 import time
 import os
 import contextlib
-import multiprocessing
 import lightgbm as lgb
 
 from supervised.algorithms.algorithm import BaseAlgorithm
@@ -35,16 +34,21 @@ class LightgbmAlgorithm(BaseAlgorithm):
         self.max_iters = 1
         self.early_stopping_rounds = additional.get("early_stopping_rounds", 50)
 
+        n_jobs = self.params.get("n_jobs", 0)
+        # 0 is the default for LightGBM to use all cores
+        if n_jobs == -1:
+            n_jobs = 0
+
         self.learner_params = {
             "boosting_type": "gbdt",
             "objective": self.params.get("objective", "binary"),
             "metric": self.params.get("metric", "binary_logloss"),
-            "num_threads": multiprocessing.cpu_count(),
             "num_leaves": self.params.get("num_leaves", 31),
             "learning_rate": self.params.get("learning_rate", 0.1),
             "feature_fraction": self.params.get("feature_fraction", 1.0),
             "bagging_fraction": self.params.get("bagging_fraction", 1.0),
             "min_data_in_leaf": self.params.get("min_data_in_leaf", 20),
+            "num_threads": n_jobs,
             "verbose": -1,
             "seed": self.params.get("seed", 1),
         }
@@ -71,7 +75,7 @@ class LightgbmAlgorithm(BaseAlgorithm):
             valid_sets=valid_sets,
             early_stopping_rounds=esr,
             evals_result=evals_result,
-            verbose_eval=False
+            verbose_eval=False,
         )
         time_1_iter = (time.time() - start_time) / 2.0
 
@@ -91,7 +95,7 @@ class LightgbmAlgorithm(BaseAlgorithm):
         y_validation=None,
         sample_weight_validation=None,
         log_to_file=None,
-        max_time=None
+        max_time=None,
     ):
         lgb_train = lgb.Dataset(
             X.to_numpy() if isinstance(X, pd.DataFrame) else X, y, weight=sample_weight
@@ -123,7 +127,7 @@ class LightgbmAlgorithm(BaseAlgorithm):
             evals_result = {}
 
             # disable for now ...
-            # boosting_rounds = self.get_boosting_rounds(lgb_train, valid_sets, esr, max_time)            
+            # boosting_rounds = self.get_boosting_rounds(lgb_train, valid_sets, esr, max_time)
 
             self.model = lgb.train(
                 self.learner_params,
@@ -147,6 +151,9 @@ class LightgbmAlgorithm(BaseAlgorithm):
                 )
                 result.to_csv(log_to_file, index=False, header=False)
 
+    def is_fitted(self):
+        return self.model is not None
+
     def predict(self, X):
         self.reload()
         return self.model.predict(X.to_numpy() if isinstance(X, pd.DataFrame) else X)
@@ -165,25 +172,6 @@ class LightgbmAlgorithm(BaseAlgorithm):
         self.model_file_path = model_file_path
         self.model = lgb.Booster(model_file=model_file_path)
 
-    def get_params(self):
-        json_desc = {
-            "library_version": self.library_version,
-            "algorithm_name": self.algorithm_name,
-            "algorithm_short_name": self.algorithm_short_name,
-            "uid": self.uid,
-            "params": self.params,
-        }
-        return json_desc
-
-    def set_params(self, json_desc):
-        self.library_version = json_desc.get("library_version", self.library_version)
-        self.algorithm_name = json_desc.get("algorithm_name", self.algorithm_name)
-        self.algorithm_short_name = json_desc.get(
-            "algorithm_short_name", self.algorithm_short_name
-        )
-        self.uid = json_desc.get("uid", self.uid)
-        self.params = json_desc.get("params", self.params)
-
     def get_metric_name(self):
         metric = self.params.get("metric")
         if metric is None:
@@ -251,7 +239,7 @@ classification_multi_default_params = {
 
 lgbr_params = copy.deepcopy(lgbm_bin_params)
 lgbr_params["objective"] = ["regression"]
-lgbr_params["metric"] = ["rmse"]
+lgbr_params["metric"] = ["rmse", "mae", "mape"]
 
 AlgorithmsRegistry.add(
     BINARY_CLASSIFICATION,
