@@ -41,13 +41,16 @@ class RandomForestAlgorithm(SklearnTreesEnsembleClassifierAlgorithm):
             max_depth=params.get("max_depth", 6),
             min_samples_split=params.get("min_samples_split", 4),
             warm_start=True,
-            n_jobs=-1,
+            n_jobs=params.get("n_jobs", -1),
             random_state=params.get("seed", 1),
         )
 
     def file_extension(self):
         return "random_forest"
 
+    def get_metric_name(self):
+        return "logloss"
+
 
 class RandomForestRegressorAlgorithm(SklearnTreesEnsembleRegressorAlgorithm):
 
@@ -71,13 +74,16 @@ class RandomForestRegressorAlgorithm(SklearnTreesEnsembleRegressorAlgorithm):
             max_depth=params.get("max_depth", 6),
             min_samples_split=params.get("min_samples_split", 4),
             warm_start=True,
-            n_jobs=-1,
+            n_jobs=params.get("n_jobs", -1),
             random_state=params.get("seed", 1),
         )
 
     def file_extension(self):
         return "random_forest"
 
+    def get_metric_name(self):
+        return "rmse"
+
 
 # For binary classification target should be 0, 1. There should be no NaNs in target.
 rf_params = {
