@@ -48,7 +48,6 @@ class XgbAlgorithm(BaseAlgorithm):
         self.boosting_rounds = additional.get("max_rounds", 10000)
         self.max_iters = 1
         self.early_stopping_rounds = additional.get("early_stopping_rounds", 50)
-
         self.learner_params = {
             "tree_method": "hist",
             "booster": "gbtree",
@@ -59,6 +58,7 @@ class XgbAlgorithm(BaseAlgorithm):
             "min_child_weight": self.params.get("min_child_weight", 1),
             "subsample": self.params.get("subsample", 0.8),
             "colsample_bytree": self.params.get("colsample_bytree", 0.8),
+            "n_jobs": self.params.get("n_jobs", -1),
             # "silent": self.params.get("silent", 1),
             "seed": self.params.get("seed", 1),
         }
@@ -105,7 +105,7 @@ class XgbAlgorithm(BaseAlgorithm):
         y_validation=None,
         sample_weight_validation=None,
         log_to_file=None,
-        max_time=None
+        max_time=None,
     ):
         dtrain = xgb.DMatrix(
             X.to_numpy() if isinstance(X, pd.DataFrame) else X,
@@ -145,9 +145,9 @@ class XgbAlgorithm(BaseAlgorithm):
             # callbacks=[time_constraint] # callback slows down by factor ~8
         )
 
-        #dump_list = self.model.get_dump()
-        #num_trees = len(dump_list)
-        #print(self.model.best_ntree_limit, num_trees)
+        # dump_list = self.model.get_dump()
+        # num_trees = len(dump_list)
+        # print(self.model.best_ntree_limit, num_trees)
 
         if log_to_file is not None:
             metric_name = list(evals_result["train"].keys())[0]
@@ -175,6 +175,9 @@ class XgbAlgorithm(BaseAlgorithm):
             self.model.load_model(tmp.name)
         """
 
+    def is_fitted(self):
+        return self.model is not None
+
     def predict(self, X):
         self.reload()
 
@@ -201,26 +204,6 @@ class XgbAlgorithm(BaseAlgorithm):
         self.model.load_model(model_file_path)
         self.model_file_path = model_file_path
 
-    def get_params(self):
-        return {
-            "library_version": self.library_version,
-            "algorithm_name": self.algorithm_name,
-            "algorithm_short_name": self.algorithm_short_name,
-            "uid": self.uid,
-            "params": self.params,
-            "best_ntree_limit": self.best_ntree_limit,
-        }
-
-    def set_params(self, json_desc):
-        self.library_version = json_desc.get("library_version", self.library_version)
-        self.algorithm_name = json_desc.get("algorithm_name", self.algorithm_name)
-        self.algorithm_short_name = json_desc.get(
-            "algorithm_short_name", self.algorithm_short_name
-        )
-        self.uid = json_desc.get("uid", self.uid)
-        self.params = json_desc.get("params", self.params)
-        self.best_ntree_limit = json_desc.get("best_ntree_limit", self.best_ntree_limit)
-
     def file_extension(self):
         return "xgboost"
 
@@ -236,6 +219,11 @@ class XgbAlgorithm(BaseAlgorithm):
             return "logloss"
         elif metric == "rmse":
             return "rmse"
+        elif metric == "mae":
+            return "mae"
+        elif metric == "mape":
+            return "mape"
+
         return None
 
 
@@ -262,7 +250,7 @@ classification_bin_default_params = {
 
 xgb_regression_params = dict(xgb_bin_class_params)
 xgb_regression_params["objective"] = ["reg:squarederror"]
-xgb_regression_params["eval_metric"] = ["rmse", "mae"]
+xgb_regression_params["eval_metric"] = ["rmse", "mae", "mape"]
 xgb_regression_params["max_depth"] = [4, 5, 6, 7, 8, 9]
 
 
