@@ -26,6 +26,7 @@ from supervised.callbacks.learner_time_constraint import LearnerTimeConstraint
 from supervised.callbacks.total_time_constraint import TotalTimeConstraint
 from supervised.ensemble import Ensemble
 from supervised.exceptions import AutoMLException
+from supervised.exceptions import NotTrainedException
 from supervised.model_framework import ModelFramework
 from supervised.preprocessing.exclude_missing_target import ExcludeRowsMissingTarget
 from supervised.tuner.data_info import DataInfo
@@ -41,6 +42,8 @@ from supervised.utils.data_validation import (
     check_positive_integer,
     check_greater_than_zero_integer,
     check_bool,
+    check_greater_than_zero_integer_or_float,
+    check_integer,
 )
 
 logger = logging.getLogger(__name__)
@@ -82,7 +85,7 @@ class BaseAutoML(BaseEstimator, ABC):
         self._confusion_matrix = None
         self._X_path, self._y_path = None, None
         self._data_info = None
-        self._model_paths = []
+        self._model_subpaths = []
         self._stacked_models = None
         self._fit_level = None
         self._start_time = time.time()
@@ -93,6 +96,9 @@ class BaseAutoML(BaseEstimator, ABC):
         self.tuner = None
         self._boost_on_errors = None
         self._kmeans_features = None
+        self._mix_encoding = None
+        self._max_single_prediction_time = None
+        self._n_jobs = -1
 
     def _get_tuner_params(
         self, start_random_models, hill_climbing_steps, top_models_to_improve
@@ -118,7 +124,7 @@ class BaseAutoML(BaseEstimator, ABC):
         try:
             params = json.load(open(os.path.join(path, "params.json")))
 
-            self._model_paths = params["saved"]
+            self._model_subpaths = params["saved"]
             self._mode = params.get("mode", self._mode)
             self._ml_task = params.get("ml_task", self._ml_task)
             self._results_path = params.get("results_path", self._results_path)
@@ -152,36 +158,48 @@ class BaseAutoML(BaseEstimator, ABC):
             )
             self._boost_on_errors = params.get("boost_on_errors", self._boost_on_errors)
             self._kmeans_features = params.get("kmeans_features", self._kmeans_features)
+            self._mix_encoding = params.get("mix_encoding", self._mix_encoding)
+            self._max_single_prediction_time = params.get(
+                "max_single_prediction_time", self._max_single_prediction_time
+            )
+            self._n_jobs = params.get("n_jobs", self._n_jobs)
             self._random_state = params.get("random_state", self._random_state)
             stacked_models = params.get("stacked")
 
+            load_on_predict = params.get("load_on_predict")
+            self._fit_level = params.get("fit_level")
+            lazy_load = not (
+                self._fit_level is not None and self._fit_level == "finished"
+            )
+            load_models = self._model_subpaths
+            if load_on_predict is not None and self._fit_level == "finished":
+                load_models = load_on_predict
             models_map = {}
-            for model_path in self._model_paths:
-                if model_path.endswith("Ensemble") or model_path.endswith(
+
+            for model_subpath in load_models:
+                if model_subpath.endswith("Ensemble") or model_subpath.endswith(
                     "Ensemble_Stacked"
                 ):
-                    ens = Ensemble.load(model_path, models_map)
+                    ens = Ensemble.load(path, model_subpath, models_map)
                     self._models += [ens]
                     models_map[ens.get_name()] = ens
                 else:
-                    m = ModelFramework.load(model_path)
+                    m = ModelFramework.load(path, model_subpath, lazy_load)
                     self._models += [m]
                     models_map[m.get_name()] = m
 
-            if stacked_models is not None:
+            best_model_name = params.get("best_model")
+            self._best_model = None
+            if best_model_name is not None:
+                self._best_model = models_map.get(best_model_name)
+
+            if stacked_models is not None and (
+                self._best_model._is_stacked or self._fit_level != "finished"
+            ):
                 self._stacked_models = []
                 for stacked_model_name in stacked_models:
                     self._stacked_models += [models_map[stacked_model_name]]
 
-            best_model_fname = os.path.join(path, "best_model.txt")
-            if os.path.exists(best_model_fname):
-                best_model_name = None
-                with open(best_model_fname, "r") as fin:
-                    best_model_name = fin.read().rstrip()
-                self._best_model = models_map[best_model_name]
-            else:
-                self._best_model = None
-
             data_info_path = os.path.join(path, "data_info.json")
             self._data_info = json.load(open(data_info_path))
             self.n_features_in_ = self._data_info["n_features"]
@@ -189,11 +207,12 @@ class BaseAutoML(BaseEstimator, ABC):
             if "n_classes" in self._data_info:
                 self.n_classes = self._data_info["n_classes"]
 
-            self._fit_level = "finished"
         except Exception as e:
             raise AutoMLException(f"Cannot load AutoML directory. {str(e)}")
 
-    def get_leaderboard(self, filter_random_feature=False):
+    def get_leaderboard(
+        self, filter_random_feature=False, original_metric_values=False
+    ):
         ldb = {
             "name": [],
             "model_type": [],
@@ -201,6 +220,8 @@ class BaseAutoML(BaseEstimator, ABC):
             "metric_value": [],
             "train_time": [],
         }
+        if self._max_single_prediction_time is not None:
+            ldb["single_prediction_time"] = []
         for m in self._models:
             # filter model with random feature
             if filter_random_feature and "RandomFeature" in m.get_name():
@@ -210,35 +231,58 @@ class BaseAutoML(BaseEstimator, ABC):
             ldb["metric_type"] += [self._eval_metric]
             ldb["metric_value"] += [m.get_final_loss()]
             ldb["train_time"] += [np.round(m.get_train_time(), 2)]
+            if self._max_single_prediction_time is not None:
+                ldb["single_prediction_time"] += [
+                    np.round(m._single_prediction_time, 4)
+                ]
 
         ldb = pd.DataFrame(ldb)
         # need to add argument for sorting
         # minimize_direction = m.get_metric().get_minimize_direction()
         # ldb = ldb.sort_values("metric_value", ascending=minimize_direction)
 
+        if original_metric_values:
+            if Metric.optimize_negative(self._eval_metric):
+                ldb["metric_value"] *= -1.0
+
         return ldb
 
-    def keep_model(self, model, model_path):
+    def keep_model(self, model, model_subpath):
         if model is None:
             return
+
+        if self._max_single_prediction_time is not None:
+            # let's check the prediction time ...
+            # load 2x because of model reloading during the training
+            for _ in range(2):
+                start_time = time.time()
+                self._base_predict(self._one_sample, model)
+                model._single_prediction_time = (
+                    time.time() - start_time
+                )  # prediction time on single sample
+            # again release learners from models
+            if "Ensemble" not in model.get_type():
+                model.release_learners()
+
         self._models += [model]
-        self._model_paths += [model_path]
+        self._model_subpaths += [model_subpath]
         self.select_and_save_best()
 
-        self.verbose_print(
-            "{} {} {} trained in {} seconds".format(
-                model.get_name(),
-                self._eval_metric,
-                np.round(model.get_final_loss(), 6),
-                np.round(model.get_train_time(), 2),
-            )
+        sign = -1.0 if Metric.optimize_negative(self._eval_metric) else 1.0
+        msg = "{} {} {} trained in {} seconds".format(
+            model.get_name(),
+            self._eval_metric,
+            np.round(sign * model.get_final_loss(), 6),
+            np.round(model.get_train_time(), 2),
         )
+        if model._single_prediction_time is not None:
+            msg += f" (1-sample predict time {np.round(model._single_prediction_time,4)} seconds)"
+        self.verbose_print(msg)
         self._time_ctrl.log_time(
             model.get_name(), model.get_type(), self._fit_level, model.get_train_time()
         )
-        
+
         self.tuner.add_key(model)
-        
 
     def create_dir(self, model_path):
         if not os.path.exists(model_path):
@@ -266,7 +310,8 @@ class BaseAutoML(BaseEstimator, ABC):
             logger.info(f"Cannot train {params['name']} because of the time constraint")
             return False
         # let's create directory to log all training artifacts
-        model_path = os.path.join(self._results_path, params["name"])
+        results_path, model_subpath = self._results_path, params["name"]
+        model_path = os.path.join(results_path, model_subpath)
         self.create_dir(model_path)
 
         # prepare callbacks
@@ -279,12 +324,12 @@ class BaseAutoML(BaseEstimator, ABC):
         if self._total_time_limit is not None:
             k_folds = self._validation_strategy.get("k_folds", 1.0)
             at_least_algorithms = 10.0
-            
-            max_time_for_learner = max(self._total_time_limit / k_folds / at_least_algorithms, 60) 
-            
-        #print("max_time_for_learner --->", max_time_for_learner)
-        params["max_time_for_learner"] = max_time_for_learner
 
+            max_time_for_learner = max(
+                self._total_time_limit / k_folds / at_least_algorithms, 60
+            )
+
+        params["max_time_for_learner"] = max_time_for_learner
 
         total_time_constraint = TotalTimeConstraint(
             {
@@ -306,13 +351,13 @@ class BaseAutoML(BaseEstimator, ABC):
         logger.info(
             f"Train model #{len(self._models)+1} / Model name: {params['name']}"
         )
-        mf.train(model_path)
+        mf.train(results_path, model_subpath)
 
         # save the model
-        mf.save(model_path)
+        mf.save(results_path, model_subpath)
 
         # and keep info about the model
-        self.keep_model(mf, model_path)
+        self.keep_model(mf, model_subpath)
         return True
 
     def verbose_print(self, msg):
@@ -323,18 +368,20 @@ class BaseAutoML(BaseEstimator, ABC):
     def ensemble_step(self, is_stacked=False):
         if self._train_ensemble and len(self._models) > 1:
 
-            ensemble_path = os.path.join(
-                self._results_path, "Ensemble_Stacked" if is_stacked else "Ensemble"
-            )
+            ensemble_subpath = "Ensemble_Stacked" if is_stacked else "Ensemble"
+            ensemble_path = os.path.join(self._results_path, ensemble_subpath)
             self.create_dir(ensemble_path)
 
             self.ensemble = Ensemble(
-                self._eval_metric, self._ml_task, is_stacked=is_stacked
+                self._eval_metric,
+                self._ml_task,
+                is_stacked=is_stacked,
+                max_single_prediction_time=self._max_single_prediction_time,
             )
             oofs, target, sample_weight = self.ensemble.get_oof_matrix(self._models)
             self.ensemble.fit(oofs, target, sample_weight)
-            self.ensemble.save(ensemble_path)
-            self.keep_model(self.ensemble, ensemble_path)
+            self.ensemble.save(self._results_path, ensemble_subpath)
+            self.keep_model(self.ensemble, ensemble_subpath)
             return True
         return False
 
@@ -397,6 +444,14 @@ class BaseAutoML(BaseEstimator, ABC):
             self._stacked_models[i] for i in np.argsort(scores).tolist()
         ]
 
+    def get_stacking_minimum_time_needed(self):
+        try:
+            ldb = self.get_leaderboard(filter_random_feature=True)
+            ldb = ldb.sort_values(by="metric_value", ascending=True)
+            return min(2.0 * ldb.iloc[0]["train_time"], 60)
+        except Exception as e:
+            return 60
+
     def prepare_for_stacking(self):
         # print("Stacked models ....")
         # do we have enough models?
@@ -405,8 +460,10 @@ class BaseAutoML(BaseEstimator, ABC):
         # do we have time?
         if self._total_time_limit is not None:
             time_left = self._total_time_limit - (time.time() - self._start_time)
-            # we need at least 60 seconds to do anything
-            if time_left < 60:
+            # we need some time to start stacking
+            # it should be at least 60 seconds for larger data
+            # but for small data it can be less
+            if time_left < self.get_stacking_minimum_time_needed():
                 return
         # too many classes and models
         if self._ml_task == MULTICLASS_CLASSIFICATION:
@@ -493,6 +550,9 @@ class BaseAutoML(BaseEstimator, ABC):
         if sample_weight is not None:
             self._validation_strategy["sample_weight_path"] = self._sample_weight_path
 
+        if self._max_single_prediction_time is not None:
+            self._one_sample = X.iloc[:1].copy(deep=True)
+
         self._drop_data_variables(X)
 
     def _handle_drastic_imbalance(self, X, y, sample_weight=None):
@@ -667,6 +727,17 @@ class BaseAutoML(BaseEstimator, ABC):
 
     def _apply_constraints(self):
 
+        if "Neural Network" in self._algorithms and self._n_jobs != -1:
+            self._algorithms.remove("Neural Network")
+            self.verbose_print(
+                "Neural Network algorithm was disabled because it doesn't support n_jobs parameter."
+            )
+        if "Linear" in self._algorithms and not (self.n_rows_in_ < 10000 and self.n_features_in_ < 1000):
+            self._algorithms.remove("Linear")
+            self.verbose_print(
+                "Linear algorithm was disabled."
+            )
+
         # remove algorithms in the case of multiclass
         # and too many classes and columns
         if self._ml_task == MULTICLASS_CLASSIFICATION:
@@ -732,7 +803,7 @@ class BaseAutoML(BaseEstimator, ABC):
 
         # adjust the validation if possible
         if folds_cnt >= 5.0:
-            self.verbose_print(f"Adjust validation. Remove: {self._model_paths[0]}")
+            self.verbose_print(f"Adjust validation. Remove: {self._model_subpaths[0]}")
             k_folds = 5
             if folds_cnt >= 15:
                 k_folds = 10
@@ -743,9 +814,9 @@ class BaseAutoML(BaseEstimator, ABC):
             del self._validation_strategy["train_ratio"]
             self._validation_strategy["k_folds"] = k_folds
             self.tuner._validation_strategy = self._validation_strategy
-            shutil.rmtree(self._model_paths[0], ignore_errors=True)
+            shutil.rmtree(self._model_subpaths[0], ignore_errors=True)
             del self._models[0]
-            del self._model_paths[0]
+            del self._model_subpaths[0]
             del self.tuner._unique_params_keys[0]
             self._adjust_validation = False
             cv = []
@@ -754,9 +825,7 @@ class BaseAutoML(BaseEstimator, ABC):
             if self._validation_strategy.get("stratify", False):
                 cv += ["Stratify"]
             self.select_and_save_best()  # save validation strategy
-            os.remove(
-                os.path.join(self._results_path, "best_model.txt")
-            )  # remove the file
+
             self.verbose_print(f"Validation strategy: {k_folds}-fold CV {','.join(cv)}")
         else:
             # cant stack models for train/test split
@@ -829,6 +898,9 @@ class BaseAutoML(BaseEstimator, ABC):
         self._top_models_to_improve = self._get_top_models_to_improve()
         self._boost_on_errors = self._get_boost_on_errors()
         self._kmeans_features = self._get_kmeans_features()
+        self._mix_encoding = self._get_mix_encoding()
+        self._max_single_prediction_time = self._get_max_single_prediction_time()
+        self._n_jobs = self._get_n_jobs()
         self._random_state = self._get_random_state()
 
         self._adjust_validation = False
@@ -889,6 +961,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 self._adjust_validation,
                 self._boost_on_errors,
                 self._kmeans_features,
+                self._mix_encoding,
+                self._n_jobs,
                 self._random_state,
             )
             self.tuner = tuner
@@ -979,6 +1053,11 @@ class BaseAutoML(BaseEstimator, ABC):
                         ):
                             self._set_adjusted_validation()
 
+                    except NotTrainedException as e:
+                        params["status"] = "error"
+                        self.verbose_print(
+                            params.get("name") + " not trained. " + str(e)
+                        )
                     except Exception as e:
                         self._update_errors_report(
                             params.get("name"), str(e) + "\n" + traceback.format_exc()
@@ -987,12 +1066,16 @@ class BaseAutoML(BaseEstimator, ABC):
 
                     self.save_progress(step, generated_params)
 
+            if not self._models:
+                raise AutoMLException("No models produced.")
             self._fit_level = "finished"
             self.save_progress()
+            self.select_and_save_best(show_warnings=True)
 
             self.verbose_print(
                 f"AutoML fit time: {np.round(time.time() - self._start_time,2)} seconds"
             )
+            self.verbose_print(f"AutoML best model: {self._best_model.get_name()}")
 
         except Exception as e:
             raise e
@@ -1016,17 +1099,40 @@ class BaseAutoML(BaseEstimator, ABC):
             )
             fout.write("\n\n")
 
-    def select_and_save_best(self):
+    def select_and_save_best(self, show_warnings=False):
         # Select best model based on the lowest loss
+        self._best_model = None
         if self._models:
+            model_list = [
+                m
+                for m in self._models
+                if m.is_valid() and m.is_fast_enough(self._max_single_prediction_time)
+            ]
+            if model_list:
+                self._best_model = min(
+                    model_list,
+                    key=lambda x: x.get_final_loss(),
+                )
+        # if none selected please select again and warn the user
+        if (
+            len(self._models)
+            and self._best_model is None
+            and self._max_single_prediction_time is not None
+        ):
+            if show_warnings:
+                msg = (
+                    "*" * 64
+                    + "\nThere were no model with prediction time smaller than the limit.\n"
+                    + "Please increase the prediction time for single sample,\n"
+                    + "or please to use train/test split for validation\n"
+                    + "*" * 64
+                )
+                self.verbose_print(msg)
+
             self._best_model = min(
                 [m for m in self._models if m.is_valid()],
                 key=lambda x: x.get_final_loss(),
             )
-            with open(os.path.join(self._results_path, "best_model.txt"), "w") as fout:
-                fout.write(f"{self._best_model.get_name()}")
-        else:
-            self._best_model = None
 
         with open(os.path.join(self._results_path, "params.json"), "w") as fout:
             params = {
@@ -1049,15 +1155,32 @@ class BaseAutoML(BaseEstimator, ABC):
                 "top_models_to_improve": self._top_models_to_improve,
                 "boost_on_errors": self._boost_on_errors,
                 "kmeans_features": self._kmeans_features,
+                "mix_encoding": self._mix_encoding,
+                "max_single_prediction_time": self._max_single_prediction_time,
+                "n_jobs": self._n_jobs,
                 "random_state": self._random_state,
-                "saved": self._model_paths,
+                "saved": self._model_subpaths,
+                "fit_level": self._fit_level,
             }
+            if self._best_model is not None:
+                params["best_model"] = self._best_model.get_name()
+                load_on_predict = []
+                if self._best_model._is_stacked and self._stacked_models is not None:
+                    load_on_predict += [m.get_name() for m in self._stacked_models]
+                if "Ensemble" in self._best_model.get_type():
+                    load_on_predict += [
+                        ens["model"].get_name()
+                        for ens in self._best_model.selected_models
+                    ]
+                load_on_predict += [self._best_model.get_name()]
+                params["load_on_predict"] = list(np.unique(load_on_predict))
+
             if self._stacked_models is not None:
                 params["stacked"] = [m.get_name() for m in self._stacked_models]
             fout.write(json.dumps(params, indent=4))
 
         if self._models:
-            ldb = self.get_leaderboard()
+            ldb = self.get_leaderboard(original_metric_values=True)
             ldb.to_csv(os.path.join(self._results_path, "leaderboard.csv"), index=False)
 
             # save report
@@ -1072,18 +1195,18 @@ class BaseAutoML(BaseEstimator, ABC):
                 fout.write(tabulate(ldb.values, ldb.columns, tablefmt="pipe"))
                 LeaderboardPlots.compute(ldb, self._results_path, fout)
 
-    def _check_is_fitted(self):
-        # First check if model can be loaded
-        self._check_can_load()
-        # Check if fitted
-        if self._fit_level != "finished":
+    def _base_predict(self, X, model=None):
+
+        if model is None:
+            if self._best_model is None:
+                self.load(self.results_path)
+            model = self._best_model
+
+        if model is None:
             raise AutoMLException(
                 "This model has not been fitted yet. Please call `fit()` first."
             )
 
-    def _base_predict(self, X):
-        self._check_is_fitted()
-
         X = self._build_dataframe(X)
         if not isinstance(X.columns[0], str):
             X.columns = [str(c) for c in X.columns]
@@ -1099,17 +1222,17 @@ class BaseAutoML(BaseEstimator, ABC):
         self._validate_X_predict(X)
 
         # is stacked model
-        if self._best_model._is_stacked:
+        if model._is_stacked:
             self._perform_model_stacking()
             X_stacked = self.get_stacked_data(X, mode="predict")
 
-            if self._best_model.get_type() == "Ensemble":
+            if model.get_type() == "Ensemble":
                 # Ensemble is using both original and stacked data
-                predictions = self._best_model.predict(X, X_stacked)
+                predictions = model.predict(X, X_stacked)
             else:
-                predictions = self._best_model.predict(X_stacked)
+                predictions = model.predict(X_stacked)
         else:
-            predictions = self._best_model.predict(X)
+            predictions = model.predict(X)
 
         if self._ml_task == BINARY_CLASSIFICATION:
             # need to predict the label based on predictions and threshold
@@ -1125,7 +1248,7 @@ class BaseAutoML(BaseEstimator, ABC):
                 neg_label = int(neg_label)
                 pos_label = int(pos_label)
             # assume that it is binary classification
-            predictions["label"] = predictions.iloc[:, 1] > self._best_model._threshold
+            predictions["label"] = predictions.iloc[:, 1] > model._threshold
             predictions["label"] = predictions["label"].map(
                 {True: pos_label, False: neg_label}
             )
@@ -1281,6 +1404,7 @@ class BaseAutoML(BaseEstimator, ABC):
             if self._get_mode() == "Compete":
                 return [
                     "Decision Tree",
+                    "Linear",
                     "Random Forest",
                     "Extra Trees",
                     "LightGBM",
@@ -1465,6 +1589,34 @@ class BaseAutoML(BaseEstimator, ABC):
         else:
             return deepcopy(self.kmeans_features)
 
+    def _get_mix_encoding(self):
+        """ Gets the current mix_encoding"""
+        self._validate_mix_encoding()
+        if self.mix_encoding == "auto":
+            if self._get_mode() == "Explain":
+                return False
+            if self._get_mode() == "Perform":
+                return False
+            if self._get_mode() == "Compete":
+                return True
+        else:
+            return deepcopy(self.mix_encoding)
+
+    def _get_max_single_prediction_time(self):
+        """ Gets the current max_single_prediction_time"""
+        self._validate_max_single_prediction_time()
+        if self.max_single_prediction_time is None:
+            if self._get_mode() == "Perform":
+                return 0.5  # prediction time should be under 0.5 second
+            return None
+        else:
+            return deepcopy(self.max_single_prediction_time)
+
+    def _get_n_jobs(self):
+        """ Gets the current n_jobs"""
+        self._validate_n_jobs()
+        return deepcopy(self.n_jobs)
+
     def _get_random_state(self):
         """ Gets the current random_state"""
         self._validate_random_state()
@@ -1558,6 +1710,7 @@ class BaseAutoML(BaseEstimator, ABC):
             "mse",
             "mae",
             "r2",
+            "mape",
         ]:
             raise ValueError(
                 f"Metric {self.eval_metric} is not allowed in ML task: {self._get_ml_task()}. \
@@ -1648,13 +1801,31 @@ class BaseAutoML(BaseEstimator, ABC):
         if isinstance(self.boost_on_errors, str) and self.boost_on_errors == "auto":
             return
         check_bool(self.boost_on_errors, "boost_on_errors")
-    
+
     def _validate_kmeans_features(self):
         """ Validates kmeans_features parameter"""
         if isinstance(self.kmeans_features, str) and self.kmeans_features == "auto":
             return
         check_bool(self.kmeans_features, "kmeans_features")
 
+    def _validate_mix_encoding(self):
+        """ Validates mix_encoding parameter"""
+        if isinstance(self.mix_encoding, str) and self.mix_encoding == "auto":
+            return
+        check_bool(self.mix_encoding, "mix_encoding")
+
+    def _validate_max_single_prediction_time(self):
+        """ Validates max_single_prediction_time parameter"""
+        if self.max_single_prediction_time is None:
+            return
+        check_greater_than_zero_integer_or_float(
+            self.max_single_prediction_time, "max_single_prediction_time"
+        )
+
+    def _validate_n_jobs(self):
+        """ Validates mix_encoding parameter"""
+        check_integer(self.n_jobs, "n_jobs")
+
     def _validate_random_state(self):
         """ Validates random_state parameter"""
         check_positive_integer(self.random_state, "random_state")
@@ -1789,3 +1960,36 @@ h3 {
 
         if main_readme_html is not None:
             return IFrame(main_readme_html, width, height)
+
+
+    def _need_retrain(self, X, y, sample_weight, decrease):
+        
+        metric = self._best_model.get_metric()
+
+        X, y, sample_weight = ExcludeRowsMissingTarget.transform(
+            X, y, sample_weight, warn=True
+        )
+
+        if self._ml_task == BINARY_CLASSIFICATION:
+            prediction = self._predict_proba(X)[:,1]
+        if self._ml_task == MULTICLASS_CLASSIFICATION:
+            prediction = self._predict_proba(X)
+        else:
+            prediction = self._predict(X)
+
+        sign = -1.0 if Metric.optimize_negative(metric.name) else 1.0
+
+        
+        new_score = metric(y, prediction, sample_weight)
+        old_score = self._best_model.get_final_loss()
+
+        change = np.abs((old_score - new_score) / old_score)
+
+        # always minimize the score 
+        if new_score > old_score:
+            self.verbose_print(f"Model performance decreased by {np.round(change*100.0,2)}%")
+            return change > decrease
+        else:
+            self.verbose_print(f"Model performance increased by {np.round(change*100.0,2)}%")
+            return False    
+        
\ No newline at end of file
