@@ -15,6 +15,7 @@ from supervised.model_framework import ModelFramework
 from supervised.utils.metric import Metric
 from supervised.utils.config import LOG_LEVEL
 from supervised.utils.additional_metrics import AdditionalMetrics
+from supervised.exceptions import NotTrainedException
 
 logger = logging.getLogger(__name__)
 logger.setLevel(LOG_LEVEL)
@@ -31,7 +32,11 @@ class Ensemble:
     algorithm_short_name = "Ensemble"
 
     def __init__(
-        self, optimize_metric="logloss", ml_task=BINARY_CLASSIFICATION, is_stacked=False
+        self,
+        optimize_metric="logloss",
+        ml_task=BINARY_CLASSIFICATION,
+        is_stacked=False,
+        max_single_prediction_time=None,
     ):
         self.library_version = "0.1"
         self.uid = str(uuid.uuid4())
@@ -54,6 +59,10 @@ class Ensemble:
         self._name = "Ensemble_Stacked" if is_stacked else "Ensemble"
         self._scores = []
         self.oof_predictions = None
+        self._oof_predictions_fname = None
+        self._single_prediction_time = None  # prediction time on single sample
+        self._max_single_prediction_time = max_single_prediction_time
+        self.model_prediction_time = {}
 
     def get_train_time(self):
         return self.train_time
@@ -64,6 +73,17 @@ class Ensemble:
     def is_valid(self):
         return len(self.selected_models) > 1
 
+    def is_fast_enough(self, max_single_prediction_time):
+        # dont need to check
+        if max_single_prediction_time is None:
+            return True
+
+        # no iformation about prediction time
+        if self._single_prediction_time is None:
+            return True
+
+        return self._single_prediction_time < max_single_prediction_time
+
     def get_type(self):
         prefix = ""  # "Stacked" if self._is_stacked else ""
         return prefix + self.algorithm_short_name
@@ -83,6 +103,10 @@ class Ensemble:
         if self.oof_predictions is not None:
             return self.oof_predictions.copy(deep=True)
 
+        if self._oof_predictions_fname is not None:
+            self.oof_predictions = pd.read_csv(self._oof_predictions_fname)
+            return self.oof_predictions.copy(deep=True)
+
         if self.total_best_sum.shape[1] == 1:
             tmp_df = pd.DataFrame({"prediction": self.total_best_sum["prediction"]})
             tmp_df["target"] = self.target[self.target_columns]
@@ -113,6 +137,18 @@ class Ensemble:
         # remember models, will be needed in predictions
         self.models_map = {m.get_name(): m for m in models}
 
+        if self._max_single_prediction_time is not None:
+            self.model_prediction_time = {
+                m.get_name(): m._single_prediction_time for m in models
+            }
+
+            if not [
+                m for m in models if m.is_fast_enough(self._max_single_prediction_time)
+            ]:
+                raise NotTrainedException(
+                    "Can't contruct ensemble with prediction time smaller than limit."
+                )
+
         oofs = {}
         for m in models:
             # do not use model with RandomFeature
@@ -175,12 +211,22 @@ class Ensemble:
         selected_algs_cnt = 0  # number of selected algorithms
         self.best_algs = []  # selected algoritms indices from each loop
 
+        total_prediction_time = 0
         best_sum = None  # sum of best algorihtms
         for j in range(len(oofs)):  # iterate over all solutions
             min_score = self.metric.get_maximum()
             best_model = None
             # try to add some algorithm to the best_sum to minimize metric
             for model_name in oofs.keys():
+                if (
+                    self._max_single_prediction_time
+                    and model_name in self.model_prediction_time
+                ):
+                    if (
+                        total_prediction_time + self.model_prediction_time[model_name]
+                        > self._max_single_prediction_time
+                    ):
+                        continue
                 y_ens = self._get_mean(oofs[model_name], best_sum, j + 1)
                 score = self.metric(y, y_ens, sample_weight)
 
@@ -188,8 +234,14 @@ class Ensemble:
                     min_score = score
                     best_model = model_name
 
+            if best_model is None:
+                continue
             # there is improvement, save it
-            self._scores += [min_score]
+            # save scores for plotting learning curve
+            # if we optimize negative, then we need to multiply by -1.0
+            # to save correct values in the learning curve
+            sign = -1.0 if Metric.optimize_negative(self.metric.name) else 1.0
+            self._scores += [sign * min_score]
 
             if self.metric.improvement(previous=self.best_loss, current=min_score):
                 self.best_loss = min_score
@@ -202,8 +254,20 @@ class Ensemble:
             )
             if j == selected_algs_cnt:
                 self.total_best_sum = copy.deepcopy(best_sum)
+
+            # update prediction time estimate
+            if self._max_single_prediction_time is not None:
+                total_prediction_time = np.sum(
+                    [
+                        self.model_prediction_time[name]
+                        for name in np.unique(self.best_algs)
+                    ]
+                )
         # end of main loop #
 
+        if not self.best_algs:
+            raise NotTrainedException("Ensemble wasn't fitted.")
+
         # keep oof predictions of ensemble
         self.total_best_sum /= float(selected_algs_cnt + 1)
         self.best_algs = self.best_algs[: (selected_algs_cnt + 1)]
@@ -303,12 +367,14 @@ class Ensemble:
                 {"model": il, "repeat": repeat}
             ]
 
-    def save(self, model_path):
+    def save(self, results_path, model_subpath):
+        model_path = os.path.join(results_path, model_subpath)
         logger.info(f"Save the ensemble to {model_path}")
 
         predictions = self.get_out_of_folds()
-        predictions_fname = os.path.join(model_path, f"predictions_ensemble.csv")
-        predictions.to_csv(predictions_fname, index=False)
+        predictions_fname = os.path.join(model_subpath, f"predictions_ensemble.csv")
+        self._oof_predictions_fname = os.path.join(results_path, predictions_fname)
+        predictions.to_csv(self._oof_predictions_fname, index=False)
 
         with open(os.path.join(model_path, "ensemble.json"), "w") as fout:
             ms = []
@@ -358,7 +424,8 @@ class Ensemble:
         return desc
 
     @staticmethod
-    def load(model_path, models_map):
+    def load(results_path, model_subpath, models_map):
+        model_path = os.path.join(results_path, model_subpath)
         logger.info(f"Loading ensemble from {model_path}")
 
         json_desc = json.load(open(os.path.join(model_path, "ensemble.json")))
@@ -376,5 +443,8 @@ class Ensemble:
         ensemble._is_stacked = json_desc.get("is_stacked", ensemble._is_stacked)
         predictions_fname = json_desc.get("predictions_fname")
         if predictions_fname is not None:
-            ensemble.oof_predictions = pd.read_csv(predictions_fname)
+            ensemble._oof_predictions_fname = os.path.join(
+                results_path, predictions_fname
+            )
+
         return ensemble
