@@ -67,6 +67,8 @@ class ModelFramework:
         self._additional_metrics = None
         self._threshold = None  # used only for binary classifiers
         self._max_time_for_learner = params.get("max_time_for_learner", 3600)
+        self._oof_predictions_fname = None
+        self._single_prediction_time = None  # prediction time on single sample
 
     def get_train_time(self):
         return self.train_time
@@ -112,7 +114,7 @@ class ModelFramework:
             "validation_columns": y_validation_columns,
         }
 
-    def train(self, model_path):
+    def train(self, results_path, model_subpath):
         logger.debug(f"ModelFramework.train {self.learner_params.get('model_type')}")
 
         start_time = time.time()
@@ -136,16 +138,22 @@ class ModelFramework:
                     logger.debug("Sample weight available during the training.")
 
                 # the proprocessing is done at every validation step
-                self.preprocessings += [Preprocessing(self.preprocessing_params, self.get_name(), k_fold, repeat)]
+                self.preprocessings += [
+                    Preprocessing(
+                        self.preprocessing_params, self.get_name(), k_fold, repeat
+                    )
+                ]
 
                 X_train, y_train, sample_weight = self.preprocessings[
                     -1
                 ].fit_and_transform(
                     train_data["X"], train_data["y"], train_data.get("sample_weight")
                 )
-                X_validation, y_validation, sample_weight_validation = self.preprocessings[
-                    -1
-                ].transform(
+                (
+                    X_validation,
+                    y_validation,
+                    sample_weight_validation,
+                ) = self.preprocessings[-1].transform(
                     validation_data["X"],
                     validation_data["y"],
                     validation_data.get("sample_weight"),
@@ -161,7 +169,9 @@ class ModelFramework:
                 self.callbacks.add_and_set_learner(learner)
                 self.callbacks.on_learner_train_start()
 
-                log_to_file = os.path.join(model_path, f"{learner.name}_training.log")
+                log_to_file = os.path.join(
+                    results_path, model_subpath, f"{learner.name}_training.log"
+                )
 
                 for i in range(learner.max_iters):
 
@@ -175,11 +185,11 @@ class ModelFramework:
                         y_validation,
                         sample_weight_validation,
                         log_to_file,
-                        self._max_time_for_learner
+                        self._max_time_for_learner,
                     )
 
                     if self.params.get("injected_sample_weight", False):
-                        #print("Dont use sample weight in model evaluation")
+                        # print("Dont use sample weight in model evaluation")
                         sample_weight = None
                         sample_weight_validation = None
 
@@ -204,6 +214,7 @@ class ModelFramework:
                 # end of learner iters loop
                 self.callbacks.on_learner_train_end()
 
+                model_path = os.path.join(results_path, model_subpath)
                 learner.interpret(
                     X_train,
                     y_train,
@@ -218,9 +229,7 @@ class ModelFramework:
                 )
 
                 # save learner and free the memory
-                p = os.path.join(
-                    model_path, f"{learner.name}.{learner.file_extension()}"
-                )
+                p = os.path.join(model_path, learner.get_fname())
                 learner.save(p)
                 del learner.model
                 learner.model = None
@@ -234,6 +243,12 @@ class ModelFramework:
         self.train_time = time.time() - start_time
         logger.debug("ModelFramework end of training")
 
+    def release_learners(self):
+        for learner in self.learners:
+            if learner.model is not None:
+                del learner.model
+                learner.model = None
+
     def get_metric_name(self):
         if self.metric_name is not None:
             return self.metric_name
@@ -252,6 +267,11 @@ class ModelFramework:
     def get_out_of_folds(self):
         if self.oof_predictions is not None:
             return self.oof_predictions.copy(deep=True)
+
+        if self._oof_predictions_fname is not None:
+            self.oof_predictions = pd.read_csv(self._oof_predictions_fname)
+            return self.oof_predictions.copy(deep=True)
+
         early_stopping = self.callbacks.get("early_stopping")
         if early_stopping is None:
             return None
@@ -297,10 +317,21 @@ class ModelFramework:
         return self._name
 
     def is_valid(self):
-        """ is_valid is used in Ensemble to check if it has more than 1 model in it.
-            If Ensemble has only 1 model in it, then Ensemble shouldn't be used as best model """
+        """is_valid is used in Ensemble to check if it has more than 1 model in it.
+        If Ensemble has only 1 model in it, then Ensemble shouldn't be used as best model"""
         return True
 
+    def is_fast_enough(self, max_single_prediction_time):
+        # dont need to check
+        if max_single_prediction_time is None:
+            return True
+
+        # no iformation about prediction time
+        if self._single_prediction_time is None:
+            return True
+
+        return self._single_prediction_time < max_single_prediction_time
+
     def predict(self, X):
         logger.debug("ModelFramework.predict")
 
@@ -355,24 +386,22 @@ class ModelFramework:
                 self._threshold = float(self._additional_metrics["threshold"])
         return self._additional_metrics
 
-    def save(self, model_path):
+    def save(self, results_path, model_subpath):
         start_time = time.time()
+        model_path = os.path.join(results_path, model_subpath)
         logger.info(f"Save the model {model_path}")
 
         type_of_predictions = (
             "validation" if "k_folds" not in self.validation_params else "out_of_folds"
         )
         predictions_fname = os.path.join(
-            model_path, f"predictions_{type_of_predictions}.csv"
+            model_subpath, f"predictions_{type_of_predictions}.csv"
         )
+        self._oof_predictions_fname = os.path.join(results_path, predictions_fname)
         predictions = self.get_out_of_folds()
-        predictions.to_csv(predictions_fname, index=False)
+        predictions.to_csv(self._oof_predictions_fname, index=False)
 
-        saved = []
-        for i, l in enumerate(self.learners):
-            p = os.path.join(model_path, f"{l.name}.{l.file_extension()}")
-            # l.save(p)
-            saved += [p]
+        saved = [os.path.join(model_subpath, l.get_fname()) for l in self.learners]
 
         with open(os.path.join(model_path, "framework.json"), "w") as fout:
             preprocessing = [p.to_json() for p in self.preprocessings]
@@ -446,7 +475,8 @@ class ModelFramework:
         return desc
 
     @staticmethod
-    def load(model_path):
+    def load(results_path, model_subpath, lazy_load=True):
+        model_path = os.path.join(results_path, model_subpath)
         logger.info(f"Loading model framework from {model_path}")
 
         json_desc = json.load(open(os.path.join(model_path, "framework.json")))
@@ -460,20 +490,20 @@ class ModelFramework:
         mf._is_stacked = json_desc.get("is_stacked", mf._is_stacked)
         predictions_fname = json_desc.get("predictions_fname")
         if predictions_fname is not None:
-            mf.oof_predictions = pd.read_csv(predictions_fname)
+            mf._oof_predictions_fname = os.path.join(results_path, predictions_fname)
 
         mf.learners = []
-        for learner_desc, learner_path in zip(
+        for learner_desc, learner_subpath in zip(
             json_desc.get("learners"), json_desc.get("saved")
         ):
-
-            l = AlgorithmFactory.load(learner_desc, learner_path)
+            learner_path = os.path.join(results_path, learner_subpath)
+            l = AlgorithmFactory.load(learner_desc, learner_path, lazy_load)
             mf.learners += [l]
 
         mf.preprocessings = []
         for p in json_desc.get("preprocessing"):
             ps = Preprocessing()
-            ps.from_json(p)
+            ps.from_json(p, results_path)
             mf.preprocessings += [ps]
 
         return mf
